@@ -1,11 +1,15 @@
 import json
 from typing import Any, Dict, Tuple, List
 
-from utbot_executor.deep_serialization.memory_objects import PythonSerializer
+from utbot_executor.deep_serialization.memory_objects import PythonSerializer, MemoryDump
 from utbot_executor.deep_serialization.json_converter import MemoryDumpEncoder, deserialize_memory_objects, DumpLoader
 from utbot_executor.deep_serialization.utils import PythonId
 
 
+def serialize_memory_dump(dump: MemoryDump):
+    return json.dumps({'objects': dump}, cls=MemoryDumpEncoder)
+
+
 def serialize_object(obj: Any) -> Tuple[str, str]:
     """
     Serialize one object.
@@ -14,21 +18,39 @@ def serialize_object(obj: Any) -> Tuple[str, str]:
 
     serializer = PythonSerializer()
     id_ = serializer.write_object_to_memory(obj)
-    return id_, json.dumps({'objects': serializer.memory}, cls=MemoryDumpEncoder)
+    return id_, serialize_memory_dump(serializer.memory)
 
 
-def serialize_objects(objs: List[Any]) -> Tuple[List[PythonId], str]:
+def serialize_objects(objs: List[Any], clear_visited: bool = False) -> Tuple[List[PythonId], str]:
     """
     Serialize objects with shared memory.
     Returns list of object ids and memory dump.
     """
 
     serializer = PythonSerializer()
+    if clear_visited:
+        serializer.clear_visited()
     ids = [
         serializer.write_object_to_memory(obj)
         for obj in objs
     ]
-    return ids, json.dumps({'objects': serializer.memory}, cls=MemoryDumpEncoder)
+    return ids, serialize_memory_dump(serializer.memory)
+
+
+def serialize_objects_dump(objs: List[Any], clear_visited: bool = False) -> Tuple[List[PythonId], MemoryDump, str]:
+    """
+    Serialize objects with shared memory.
+    Returns list of object ids and memory dump.
+    """
+
+    serializer = PythonSerializer()
+    if clear_visited:
+        serializer.clear_visited()
+    ids = [
+        serializer.write_object_to_memory(obj)
+        for obj in objs
+    ]
+    return ids, serializer.memory, serialize_memory_dump(serializer.memory)
 
 
 def deserialize_objects(ids: List[str], memory: str, imports: List[str]) -> Dict[str, object]:
@@ -39,4 +61,5 @@ def deserialize_objects(ids: List[str], memory: str, imports: List[str]) -> Dict
 
     memory_dump = deserialize_memory_objects(memory)
     loader = DumpLoader(memory_dump)
+    loader.add_imports(imports)
     return {python_id: loader.load_object(PythonId(python_id)) for python_id in ids}
