@@ -1,8 +1,9 @@
+import copy
 import importlib
 import json
 import logging
 import sys
-from typing import Dict, Iterable, List, Optional, Set, Union
+from typing import Dict, Iterable, Union, Set
 from utbot_executor.deep_serialization.memory_objects import (
     MemoryObject,
     ReprMemoryObject,
@@ -11,7 +12,7 @@ from utbot_executor.deep_serialization.memory_objects import (
     ReduceMemoryObject,
     MemoryDump
 )
-from utbot_executor.deep_serialization.utils import PythonId
+from utbot_executor.deep_serialization.utils import PythonId, TypeInfo
 
 
 class MemoryObjectEncoder(json.JSONEncoder):
@@ -20,8 +21,7 @@ class MemoryObjectEncoder(json.JSONEncoder):
             base_json = {
                 'strategy': o.strategy,
                 'id': o.id_value(),
-                'kind': o.kind,
-                'module': o.module,
+                'typeinfo': o.typeinfo,
                 'comparable': o.comparable,
             }
             if isinstance(o, ReprMemoryObject):
@@ -42,6 +42,11 @@ class MemoryDumpEncoder(json.JSONEncoder):
     def default(self, o):
         if isinstance(o, MemoryDump):
             return {id_: MemoryObjectEncoder().default(o) for id_, o in o.objects.items()}
+        if isinstance(o, TypeInfo):
+            return {
+                'kind': o.kind,
+                'module': o.module,
+            }
         return json.JSONEncoder.default(self, o)
 
 
@@ -51,29 +56,44 @@ def as_repr_object(dct: Dict) -> Union[MemoryObject, Dict]:
         if dct['strategy'] == 'repr':
             obj = ReprMemoryObject.__new__(ReprMemoryObject)
             obj.value = dct['value']
-            obj.kind = dct['kind']
+            obj.typeinfo = TypeInfo(
+                kind=dct['typeinfo']['kind'],
+                module=dct['typeinfo']['module']
+            )
             obj.comparable = dct['comparable']
             return obj
         if dct['strategy'] == 'list':
             obj = ListMemoryObject.__new__(ListMemoryObject)
             obj.items = dct['items']
-            obj.kind = dct['kind']
+            obj.typeinfo = TypeInfo(
+                kind=dct['typeinfo']['kind'],
+                module=dct['typeinfo']['module']
+            )
             obj.comparable = dct['comparable']
             return obj
         if dct['strategy'] == 'dict':
             obj = DictMemoryObject.__new__(DictMemoryObject)
             obj.items = dct['items']
-            obj.kind = dct['kind']
+            obj.typeinfo = TypeInfo(
+                kind=dct['typeinfo']['kind'],
+                module=dct['typeinfo']['module']
+            )
             obj.comparable = dct['comparable']
             return obj
         if dct['strategy'] == 'reduce':
             obj = ReduceMemoryObject.__new__(ReduceMemoryObject)
-            obj.constructor = dct['constructor']
+            obj.constructor = TypeInfo(
+                kind=dct['constructor']['kind'],
+                module=dct['constructor']['module'],
+            )
             obj.args = dct['args']
             obj.state = dct['state']
             obj.listitems = dct['listitems']
             obj.dictitems = dct['dictitems']
-            obj.kind = dct['kind']
+            obj.typeinfo = TypeInfo(
+                kind=dct['typeinfo']['kind'],
+                module=dct['typeinfo']['module']
+            )
             obj.comparable = dct['comparable']
             return obj
     return dct
@@ -88,30 +108,56 @@ class DumpLoader:
     def __init__(self, memory_dump: MemoryDump):
         self.memory_dump = memory_dump
         self.memory: Dict[PythonId, object] = {}  # key is new id, value is real object
+        self.dump_id_to_real_id: Dict[PythonId, PythonId] = {}
 
-    def add_syspaths(self, syspaths: Iterable[str]):
+    def reload_id(self) -> MemoryDump:
+        new_memory_objects: Dict[PythonId, MemoryObject] = {}
+        for id_, obj in self.memory_dump.objects.items():
+            new_memory_object = copy.deepcopy(obj)
+            read_id = self.dump_id_to_real_id[id_]
+            new_memory_object.obj = self.memory[read_id]
+            if isinstance(new_memory_object, ReprMemoryObject):
+                pass
+            elif isinstance(new_memory_object, ListMemoryObject):
+                new_memory_object.items = [self.dump_id_to_real_id[id_] for id_ in new_memory_object.items]
+            elif isinstance(new_memory_object, DictMemoryObject):
+                new_memory_object.items = {
+                    self.dump_id_to_real_id[id_key]: self.dump_id_to_real_id[id_value]
+                    for id_key, id_value in new_memory_object.items.items()
+                }
+            elif isinstance(new_memory_object, ReduceMemoryObject):
+                new_memory_object.args = self.dump_id_to_real_id[new_memory_object.args]
+                new_memory_object.state = self.dump_id_to_real_id[new_memory_object.state]
+                new_memory_object.listitems = self.dump_id_to_real_id[new_memory_object.listitems]
+                new_memory_object.dictitems = self.dump_id_to_real_id[new_memory_object.dictitems]
+            new_memory_objects[self.dump_id_to_real_id[id_]] = new_memory_object
+        return MemoryDump(new_memory_objects)
+
+    @staticmethod
+    def add_syspaths(syspaths: Iterable[str]):
         for path in syspaths:
             if path not in sys.path:
-                sys.path.append(path)
+                sys.path.insert(0, path)
 
-    def add_imports(self, imports: Iterable[str]):
+    @staticmethod
+    def add_imports(imports: Iterable[str]):
         for module in imports:
             for i in range(1, module.count('.') + 2):
                 submodule_name = '.'.join(module.split('.', maxsplit=i)[:i])
                 globals()[submodule_name] = importlib.import_module(submodule_name)
 
     def load_object(self, python_id: PythonId) -> object:
-        if python_id in self.memory:
-            return self.memory[python_id]
+        if python_id in self.dump_id_to_real_id:
+            return self.memory[self.dump_id_to_real_id[python_id]]
 
         dump_object = self.memory_dump.objects[python_id]
         real_object: object
         if isinstance(dump_object, ReprMemoryObject):
             real_object = eval(dump_object.value)
         elif isinstance(dump_object, ListMemoryObject):
-            if dump_object.kind == 'builtins.set':
+            if dump_object.qualname == 'builtins.set':
                 real_object = set(self.load_object(item) for item in dump_object.items)
-            elif dump_object.kind == 'builtins.tuple':
+            elif dump_object.qualname == 'builtins.tuple':
                 real_object = tuple(self.load_object(item) for item in dump_object.items)
             else:
                 real_object = [self.load_object(item) for item in dump_object.items]
@@ -121,11 +167,14 @@ class DumpLoader:
                     for key, value in dump_object.items.items()
                     }
         elif isinstance(dump_object, ReduceMemoryObject):
-            constructor = eval(dump_object.constructor)
+            constructor = eval(dump_object.constructor.qualname)
             args = self.load_object(dump_object.args)
             real_object = constructor(*args)
 
-            self.memory[PythonId(str(id(real_object)))] = real_object
+            id_ = PythonId(str(id(real_object)))
+            self.dump_id_to_real_id[python_id] = id_
+            self.memory[id_] = real_object
+
             state = self.load_object(dump_object.state)
             for field, value in state.items():
                 setattr(real_object, field, value)
@@ -138,7 +187,10 @@ class DumpLoader:
         else:
             raise TypeError(f'Invalid type {dump_object}')
 
-        self.memory[PythonId(str(id(real_object)))] = real_object
+        id_ = PythonId(str(id(real_object)))
+        self.dump_id_to_real_id[python_id] = id_
+        self.memory[id_] = real_object
+
         return real_object
 
 
