@@ -1,16 +1,15 @@
 from __future__ import annotations
 from itertools import zip_longest
-import logging
 import pickle
-from typing import Any, Dict, List, Optional, Type
+from typing import Any, Callable, Dict, List, Optional, Set, Type
 
-from utbot_executor.deep_serialization.utils import PythonId, get_type_name, get_type, has_reduce, check_comparability, get_repr, has_repr
+from utbot_executor.deep_serialization.utils import PythonId, get_kind, has_reduce, check_comparability, get_repr, \
+    has_repr, TypeInfo, get_constructor_kind
 
 
 class MemoryObject:
     strategy: str
-    kind: str
-    module: str
+    typeinfo: TypeInfo
     comparable: bool
     is_draft: bool
     deserialized_obj: object
@@ -18,8 +17,7 @@ class MemoryObject:
 
     def __init__(self, obj: object) -> None:
         self.is_draft = True
-        self.kind = get_type_name(obj) if isinstance(obj, type) else get_type(obj)
-        self.module = obj.__module__ if isinstance(obj, type) else type(obj).__module__
+        self.typeinfo = get_kind(obj)
         self.obj = obj
 
     def _initialize(self, deserialized_obj: object = None, comparable: bool = True) -> None:
@@ -36,11 +34,14 @@ class MemoryObject:
     def __repr__(self) -> str:
         if hasattr(self, 'obj'):
             return str(self.obj)
-        return self.kind
+        return str(self.typeinfo)
 
     def __str__(self) -> str:
         return str(self.obj)
 
+    @property
+    def qualname(self) -> str:
+        return self.typeinfo.qualname
 
 
 class ReprMemoryObject(MemoryObject):
@@ -85,7 +86,7 @@ class ListMemoryObject(MemoryObject):
     def __repr__(self) -> str:
         if hasattr(self, 'obj'):
             return str(self.obj)
-        return f'{self.kind}{self.items}'
+        return f'{self.typeinfo.kind}{self.items}'
 
 
 class DictMemoryObject(MemoryObject):
@@ -118,12 +119,12 @@ class DictMemoryObject(MemoryObject):
     def __repr__(self) -> str:
         if hasattr(self, 'obj'):
             return str(self.obj)
-        return f'{self.kind}{self.items}'
+        return f'{self.typeinfo.kind}{self.items}'
 
 
 class ReduceMemoryObject(MemoryObject):
     strategy: str = 'reduce'
-    constructor: str
+    constructor: TypeInfo
     args: PythonId
     state: PythonId
     listitems: PythonId
@@ -145,21 +146,34 @@ class ReduceMemoryObject(MemoryObject):
                 )
         ]
 
-        self.constructor = get_type_name(self.reduce_value[0])
-        self.args = serializer.write_object_to_memory(self.reduce_value[1])
+        constructor_kind = get_constructor_kind(self.reduce_value[0])
 
-        self.deserialized_obj = self.reduce_value[0](*serializer[self.args])
+        is_reconstructor = constructor_kind.qualname == 'copyreg._reconstructor'
+        is_user_type = len(self.reduce_value[1]) == 3 and self.reduce_value[1][1] is object and self.reduce_value[1][2] is None
+
+        callable_constructor: Callable
+        if is_reconstructor and is_user_type:
+            callable_constructor = self.reduce_value[1][0].__new__
+            self.constructor = TypeInfo('builtins', 'object.__new__')
+            self.args = serializer.write_object_to_memory([self.reduce_value[1][0]])
+        else:
+            callable_constructor = self.reduce_value[0]
+            self.constructor = constructor_kind
+            self.args = serializer.write_object_to_memory(self.reduce_value[1])
+
+        self.deserialized_obj = callable_constructor(*serializer[self.args])
 
     def initialize(self) -> None:
         serializer = PythonSerializer()
 
+        self.comparable = True  # for recursive objects
         self.state = serializer.write_object_to_memory(self.reduce_value[2])
         self.listitems = serializer.write_object_to_memory(list(self.reduce_value[3]))
         self.dictitems = serializer.write_object_to_memory(dict(self.reduce_value[4]))
 
         deserialized_obj = self.deserialized_obj
         for key, value in serializer[self.state].items():
-            object.__setattr__(deserialized_obj, key, value)
+            object.__setattr__(deserialized_obj, key, value)  # TODO: change object.__setattr__ by self.__setattr__?
         for item in serializer[self.listitems]:
             deserialized_obj.append(item)
         for key, value in serializer[self.dictitems].items():
@@ -218,9 +232,12 @@ class MemoryDump:
 
 
 class PythonSerializer:
+    instance: PythonSerializer
     memory: MemoryDump
     created: bool = False
 
+    visited: Set[PythonId] = set()
+
     providers: List[MemoryObjectProvider] = [
             ListMemoryObjectProvider,
             DictMemoryObjectProvider,
@@ -244,13 +261,21 @@ class PythonSerializer:
     def __getitem__(self, id_: PythonId) -> object:
         return self.get_by_id(id_).deserialized_obj
 
+    def clear_visited(self):
+        self.visited.clear()
+
     def write_object_to_memory(self, py_object: object) -> PythonId:
         """Save serialized py_object to memory and return id."""
 
         id_ = PythonId(str(id(py_object)))
+
+        if id_ in self.visited:
+            return id_
+
         for provider in self.providers:
             serializer = provider.get_serializer(py_object)
             if serializer is not None:
+                self.visited.add(id_)
                 mem_obj = serializer(py_object)
                 self.memory.objects[id_] = mem_obj
                 mem_obj.initialize()
