@@ -1,41 +1,61 @@
+from __future__ import annotations
+import dataclasses
 import importlib
 import pickle
-from typing import NewType
+from typing import NewType, Tuple
 
 PythonId = NewType('PythonId', str)
 
 
+@dataclasses.dataclass
+class TypeInfo:
+    module: str
+    kind: str
+
+    @property
+    def qualname(self):
+        if self.module == '' or self.module == 'builtins':
+            return self.kind
+        return f'{self.module}.{self.kind}'
+
+    @staticmethod
+    def from_str(representation: str) -> TypeInfo:
+        if '.' in representation:
+            return TypeInfo(representation.rsplit('.', 1)[0], representation.rsplit('.', 1)[1])
+        return TypeInfo('', representation)
+
+    def __str__(self):
+        return self.qualname
+
+
 def check_comparability(py_object: object, deserialized_py_object: object) -> bool:
     return py_object == deserialized_py_object
 
 
-def get_module(py_object: object) -> str:
+def get_kind(py_object: object) -> TypeInfo:
+    """Get module and name of type"""
     if py_object is None:
-        return "types"
+        return TypeInfo("types", "NoneType")
     if isinstance(py_object, type):
-        return py_object.__module__
-    return type(py_object).__module__
-
-
-def get_type(py_object: object) -> str:
-    if py_object is None:
-        return 'types.NoneType'
+        return TypeInfo(py_object.__module__, py_object.__qualname__)
     if callable(py_object):
-        return 'typing.Callable'
+        return TypeInfo("typing", "Callable")
     module = type(py_object).__module__
-    return '{module}.{name}'.format(
-        module=module,
-        name=type(py_object).__qualname__,
-    )
+    qualname = type(py_object).__qualname__
+    return TypeInfo(module, qualname)
 
 
-def get_type_name(type_: type) -> str:
-    if type_ is None:
-        return 'types.NoneType'
-    return '{module}.{name}'.format(
-        module=type_.__module__,
-        name=type_.__qualname__,
-    )
+def get_constructor_kind(py_object: object) -> TypeInfo:
+    """Get module and name of objcet"""
+    if py_object is None:
+        return TypeInfo("types", "NoneType")
+    if isinstance(py_object, type):
+        return TypeInfo(py_object.__module__, py_object.__qualname__)
+    if callable(py_object):
+        return TypeInfo(py_object.__module__, py_object.__qualname__)
+    module = type(py_object).__module__
+    qualname = type(py_object).__qualname__
+    return TypeInfo(module, qualname)
 
 
 def has_reduce(py_object: object) -> bool:
@@ -54,7 +74,7 @@ def has_reduce(py_object: object) -> bool:
 
 def get_repr(py_object: object) -> str:
     if isinstance(py_object, type):
-        return get_type_name(py_object)
+        return str(get_kind(py_object))
     if isinstance(py_object, float):
         if repr(py_object) == 'nan':
             return "float('nan')"
@@ -69,7 +89,7 @@ def get_repr(py_object: object) -> str:
 
 
 def check_eval(py_object: object) -> bool:
-    module = get_module(py_object)
+    module = get_kind(py_object).module
     globals()[module] = importlib.import_module(module)
     try:
         eval(get_repr(py_object))
@@ -80,7 +100,7 @@ def check_eval(py_object: object) -> bool:
 
 def has_repr(py_object: object) -> bool:
     reprable_types = [
-            None,
+            type(None),
             int,
             float,
             bytes,
