@@ -1,15 +1,16 @@
 import dataclasses
 import json
-from typing import Dict, Union
+from typing import Dict, List, Union
 
 
 @dataclasses.dataclass
 class ExecutionRequest:
     function_name: str
     function_module: str
-    imports: list[str]
-    syspaths: list[str]
-    arguments_ids: list[str]
+    imports: List[str]
+    syspaths: List[str]
+    arguments_ids: List[str]
+    kwarguments_ids: Dict[str, str]
     serialized_memory: str
     filepath: str
 
@@ -22,12 +23,14 @@ class ExecutionResponse:
 class ExecutionSuccessResponse(ExecutionResponse):
     status: str
     is_exception: bool
-    statements: list[int]
-    missed_statements: list[int]
+    statements: List[int]
+    missed_statements: List[int]
+    state_init: str
     state_before: str
     state_after: str
-    args_ids: list[str]
-    kwargs_ids: list[str]
+    diff_ids: List[str]
+    args_ids: List[str]
+    kwargs_ids: Dict[str, str]
     result_id: str
 
 
@@ -44,6 +47,7 @@ def as_execution_result(dct: Dict) -> Union[ExecutionRequest, Dict]:
             'imports',
             'syspaths',
             'argumentsIds',
+            'kwargumentsIds',
             'serializedMemory',
             'filepath'
             }:
@@ -53,6 +57,7 @@ def as_execution_result(dct: Dict) -> Union[ExecutionRequest, Dict]:
                 dct['imports'],
                 dct['syspaths'],
                 dct['argumentsIds'],
+                dct['kwargumentsIds'],
                 dct['serializedMemory'],
                 dct['filepath'],
                 )
@@ -71,8 +76,10 @@ class ResponseEncoder(json.JSONEncoder):
                 "isException": o.is_exception,
                 "statements": o.statements,
                 "missedStatements": o.missed_statements,
+                "stateInit": o.state_init,
                 "stateBefore": o.state_before,
                 "stateAfter": o.state_after,
+                "diffIds": o.diff_ids,
                 "argsIds": o.args_ids,
                 "kwargsIds": o.kwargs_ids,
                 "resultId": o.result_id,
@@ -84,5 +91,6 @@ class ResponseEncoder(json.JSONEncoder):
             }
         return json.JSONEncoder.default(self, o)
 
+
 def serialize_response(response: ExecutionResponse) -> str:
     return json.dumps(response, cls=ResponseEncoder)
